<?php

namespace Emulator\Network\Game;

use Emulator\Core\Logging\Logger;
use Emulator\Encoding\ByteEncoding;
use Emulator\Messages\Incoming\ClientMessage;
use Emulator\Messages\MessageHandler;
use Emulator\Network\Game\Sessions\Session;
use OpenSwoole\WebSocket\Server;
use OpenSwoole\Http\Request;
use OpenSwoole\WebSocket\Frame;

class GameNetworkServer {
    private static ?GameNetworkServer $instance = null;
    private array $sessions;

    public function start(int $port): void {
        $server = new Server("127.0.0.1", $port);
        $server->on('Start', [$this, 'onStart']);
        $server->on('Open', [$this, 'onOpen']);
        $server->on('Close', [$this, 'onClose']);
        $server->on('Disconnect', [$this, 'onDisconnect']);
        $server->on('Message', [$this, 'onMessage']);
        $server->start();
    }

    public static function getInstance(): GameNetworkServer {
        if (self::$instance === null) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    function onStart(Server $server): void {
        Logger::info("Websocket server is started at $server->host:$server->port");
    }

    function onOpen(Server $server, Request $request): void {
        $this->sessions[$request->fd] = new Session($server, $request);
    }

    function onClose(Server $server, int $fd): void {
        Logger::info("Connection #$fd has been closed");
        unset($this->sessions[$fd]);
    }

    function onDisconnect(Server $server, int $fd): void {
        Logger::info("Connection #$fd has disconnected");
        unset($this->sessions[$fd]);
    }

    function onMessage(Server $server, Frame $frame): void {
        $data = unpack('C*', $frame->data);

        if (count($data) >= 6) {
            $length = ByteEncoding::getInt32(array_slice($data, 1, 4));
            $packet = array_slice($data, 4, $length);

            MessageHandler::getInstance()->handlePacket($this->sessions[$frame->fd], new ClientMessage($packet));
        }
    }
}